import requests

API_URL = 'https://api.spurwing.io/api/v2/'

def get_appointment_types(provider_id=None, page_size=None, offset=None):
  return HTTP('GET', 'appointment_types.json',
    params = {
      'provider_id':provider_id,
      'page_size': page_size,
      'offset': offset })

def get_days_available(provider_id, appointment_type_id, date_from_month=None, organization_level=None, timezone=None):
  return HTTP('GET', 'bookings/days_available.json',
    params={
      'provider_id':provider_id,
      'appointment_type_id': appointment_type_id,
      'date_from_month': date_from_month,
      'timezone': timezone,
      'organization_level': organization_level })

def get_slots_available(provider_id, appointment_type_id, start_date, end_date, organization_level=None, timezone=None):
  return HTTP('GET', 'bookings/slots_available.json',
    params = {
      'provider_id':provider_id,
      'appointment_type_id': appointment_type_id,
      'start_date': start_date,
      'end_date': end_date,
      'timezone': timezone,
      'organization_level': organization_level })

def complete_booking(provider_id, appointment_type_id, email, first_name, last_name, date=None, contact_type=None, appointment_id=None, appointment_location_id=None, timezone=None, video_chat_url=None):
  return HTTP('POST', 'bookings/complete_booking.json',
    data={
      'provider_id':provider_id,
      'appointment_type_id': appointment_type_id,
      'date': date,
      'timezone': timezone,
      'first_name': first_name,
      'last_name': last_name,
      'email': email,
      'appointment_id': appointment_id,
      'appointment_location_id': appointment_location_id,
      'video_chat_url': video_chat_url,
      'contact_type': contact_type })

def list_appointments(authorization, page_size, offset, appointment_category=None, load_attendees=None, load_providers=None, load_appointment_type=None):
  return HTTP('GET', 'appointments',
    params={
      'page_size':page_size,
      'offset': offset,
      'appointment_category': appointment_category,
      'load_attendees': load_attendees,
      'load_providers': load_providers,
      'load_appointment_type': load_appointment_type },
    headers={ 'authorization': 'Bearer ' + authorization })

def delete_appointment(authorization, appointment_id):
  return HTTP('DELETE', 'appointments/' + appointment_id, headers={ 'authorization': 'Bearer ' + authorization })


def HTTP(method, endpoint, params=None, data=None, headers=None):
  url = API_URL + endpoint
  
  if (method == 'GET'):    resp = requests.get(url,    params=params, headers=headers)
  elif method == 'POST':   resp = requests.post(url,   params=params, headers=headers, data=data,)
  elif method == 'PUT':    resp = requests.put(url,    params=params, headers=headers, data=data,)
  elif method == 'DELETE': resp = requests.delete(url, params=params, headers=headers)

  if resp.status_code == 200: return resp.json()
  else: raise Exception({'status':resp.status, 'text':resp.statusText, 'url':url})



from config import KEY, PID
import Client as sp

def runner(func):
  def wrapper():
    try:
      print(func.__name__, 'STARTED')
      func()
      print('\n' + func.__name__, 'PASSED')
    except Exception as ex:
      print('\n' + func.__name__, 'FAILED')
      raise ex
  return wrapper

@runner
def test_1():
  A = sp.get_appointment_types(PID)
  log(A)
  assert len(A) == 3

  appointment_type_id=A[0]['id']

  B = sp.get_days_available(PID, appointment_type_id)
  log(B)
  assert len(B['days_available']) >= 1

  C = sp.get_slots_available(PID, appointment_type_id, dateNow(), dateTomorrow())
  log(C)
  assert len(C['slots_available']) >= 10
  slot = C['slots_available'][5]['date']

  D = sp.complete_booking(PID, appointment_type_id, 'ilya2@nevolin.be', 'Ilya', 'Nevo', date=slot, contact_type='My Contact Type')
  log(D)
  assert 'appointment' in D

  E = sp.list_appointments(KEY, 1000, 0)
  log(E)
  assert 'data' in E
  assert 'appointments' in E['data']
  assert E['data']['appointmentsCount'] >= 1

  apid = D['appointment']['id']
  F = sp.delete_appointment(KEY, apid)
  log(F)
  assert 'data' in F
  assert 'appointment' in F['data']
  assert F['data']['appointment']['id'] == apid
  assert len(F['errors']) == 0

  
@runner
def test_2():
  A = sp.list_appointments(KEY, 1000, 0)
  log(A)
  assert len(A) == 3
  assert 'data' in A
  assert 'appointments' in A['data']


from datetime import date, timedelta
def dateNow():
  return date.today().strftime("%Y/%m/%d")
def dateTomorrow():
  dt = date.today() + timedelta(days=1) 
  return dt.strftime("%Y/%m/%d")

def log(obj):
  # print(obj)
  print('.', end='')

if __name__ == "__main__":
  lc = dict.fromkeys(vars())
  for item in lc:
    if 'test_' in item:
      vars()[item]()


from setuptools import setup, find_packages
from setuptools.command.build_py import build_py as build_py_orig

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


class build_py(build_py_orig):
  def find_package_modules(self, package, package_dir):
    modules = super().find_package_modules(package, package_dir)
    included = []
    for (pkg, filename, filepath) in modules:
      if filename in ['config']:
        continue
      included.append((pkg, filename, filepath))
    return included

setup(
   name='Spurwing',
   version='1.0.5',
   author='Ilya Nevolin',
   author_email='ilya@spurwing.io',
   packages=find_packages(),
   cmdclass={'build_py': build_py},
   url='https://github.com/Spurwing/Spurwing-API-Python-Library/',
   license='MIT License (MIT)',
   description='Python Library for Spurwing API',
   long_description=long_description,
   long_description_content_type="text/markdown",
   install_requires=[
       "requests",
   ],
)